##!/usr/bin/perl -w
#
# xbt-edf.pl  Traitement des fichiers XBT Sippican.  
# Appele par script Matlab dataGUI
# Utilise les fichiers d'extensions .EDF
# Cette version 1.1 genere les 3 fichiers d'extentions .hdr, _xbt et _xbt.xml par defaut
#  
# $Id$
#

use strict;

use Time::Local;
use File::Basename;
use Data::Dumper;
use Getopt::Long;
use Oceano::Seawater;
use Oceano::Convert;
use Oceano::Seabird;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION       = '1.1';
our $AUTHOR        = 'J Grelet PIRATA-FR19 cruise - june 2009';
my  $DEBUG         = 0;
my  $ECHO;

# surcharge par la valeurs saisies sur la ligne de commande
my  $version;
my  $software         = "WinMK21";
my  $software_version = "V2.6.1";
my  $type             = "MK-21";  
my  $sn               = "01125";
my  $validation_comment = "Extraction ralise sur fichier .edf, visualisee avec datagui";
my  $institute;
my  $pi;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE      = "AMMA";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "14/06/2009";
my  $date_fin      = "23/07/2009";

my  $help;
my  $xml           = 1;       # par defaut, sortie XML activee
my  $ascii         = 1;       # par defaut, sortie ASCII activee
my  $odv           = undef;   # par defaut, sortie ODV desactivee
my  $dtd           = undef;

my  $code          = -1;      # code pour l'entete
my  $S             = 35;
my  $C;
my  $hdr_file;
my  $ascii_file;
my  $xml_file;
my  $odv_file;
my  $type_odv = "C" ;
my  $bottom_depth = 0;
my  %file;
my  $key;

my ($cycle_mesure, $plateforme, $probe) = undef;
my ($profil, $depth, $T, $sigmateta, $sndvel);
my ($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my ($long, $long_pos, $long_deg, $long_min, $long_hemi);
my ($julien, $date);
my $year_ref = undef;
# mettre a jour le decodage des entete ASAP
my ($dummy,$dpth);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: xbt-edf-fr18.pl [options] <rep/files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cycle_mesure=<name>  give cruise name\n";
  print STDERR   "    --plateforme=<name>    ship or plateforme name\n";
  print STDERR   "    --date_debut=JJ/MM/YYYY   starting date from cycle_mesure\n";
  print STDERR   "    --date_fin  =JJ/MM/YYYY   end date from cycle_mesure\n";

  print STDERR   "    --institute=<name>     institute name\n";
  print STDERR   "    --code_oopc=<value>    processing code\n";
  print STDERR   "    --pi=<pi_name> \n";
  print STDERR   "    --output=<file_name>   filename without extension\n";
  print STDERR   "    --local                use local DTD\n";
  print STDERR   "    --type=<instrument_type> \n";
  print STDERR   "    --sn=<instrument_serial_number> \n";
  print STDERR   "    --software=<software_name> \n";
  print STDERR   "    --software_version=<software_version> \n";
  print STDERR   "    --ascii                only ASCII output\n";
  print STDERR   "    --xml                  only XML output\n";
	print STDERR   "    --odv                  only ODV output\n";
  print STDERR   "Default is ASCII & XML output\n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "XBT example:\n------------\n\$ perl xbt-edf-fr18.pl  --cycle_mesure=PIRATAZ-FR19 --institute=IRD --plateforme=\"ANTEA\" --type=MK-21 --pi=Bourles --date_debut=14/06/2009 --date_fin=23/07/2009 --echo --local data/*.EDF\n\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cycle_mesure=s"     => \$cycle_mesure,    
               "type=s"             => \$type,  
	       "sn=s"               => \$sn,
               "software=s"         => \$software,  
               "software_version=s" => \$software_version,  
               "plateforme=s"       => \$plateforme,  
               "date_debut=s"       => \$date_debut,  
               "date_fin=s"         => \$date_fin,  	       
               "pi=s"               => \$pi,
               "code_oopc=s"        => \$CODE,  
               "ascii"              => \$ascii,  
               "xml"                => \$xml,  
               "odv"                => \$odv,	 
               "debug=i"            => \$DEBUG,  
               "echo"               => \$ECHO,  
               "institute=s"        => \$institute,  
               "local"              => \$dtd,  
               "version"            => \$version,  
               "help"               => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;	
  # permet d'activer les 2 options XML et ASCII sur la ligne de commande
  $xml   = undef if $ascii && !defined $xml;   
  $ascii = undef if $xml && !defined $ascii;  
  # si ODV definie, XML et ASCII desactive	
  $ascii = $xml = undef if (defined $odv);
}

=pod
#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi)=@_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m)=@_;
  my $tmp = (($h * 60) + $m ) / 1440;
  return( $jj + $tmp ); 
}
=cut

#------------------------------------------------------------------------------
# entete XML
#------------------------------------------------------------------------------
sub entete_xml { 
  my $today = &dateFormat("now", "%d/%m/%Y");
  
  print  XML_FILE '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  # les commentaires ne sont pas accepts par XML Toolbox Matlab de Geodise
  if ( defined $dtd ) {
    print  XML_FILE '<!DOCTYPE OCEANO SYSTEM "/M/ACQUISIT/oceano.dtd">' . "\n";
    #print  XML_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://antea/acquisit/oceano.dtd">' . "\n";
  } else {  
    print  XML_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://www.brest.ird.fr/us191/database/oceano.dtd">' . "\n";
  }
  print  XML_FILE '<OCEANO TYPE="PROFIL">' . "\n";
  print  XML_FILE "  <ENTETE>\n";
  print  XML_FILE "    <PLATEFORME>\n";
  print  XML_FILE "      <LIBELLE>$plateforme</LIBELLE>\n";  
  print  XML_FILE "    </PLATEFORME>\n";
  print  XML_FILE "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  XML_FILE "      <LIBELLE>$cycle_mesure</LIBELLE>\n";  
  print  XML_FILE "      <DATE_DEBUT>$date_debut</DATE_DEBUT>\n";  
  print  XML_FILE "      <DATE_FIN>$date_fin</DATE_FIN>\n";  
  print  XML_FILE "      <INSTITUT>$institute</INSTITUT>\n";  
  print  XML_FILE "      <RESPONSABLE>$pi</RESPONSABLE>\n"; 
  print  XML_FILE "      <ACQUISITION LOGICIEL=\"$software\" VERSION=\"$software_version\"></ACQUISITION>\n"; 
  print  XML_FILE "      <TRAITEMENT LOGICIEL=\"$0\" VERSION=\"$VERSION\"></TRAITEMENT>\n"; 
  print  XML_FILE "      <VALIDATION LOGICIEL=\"datagui\" VERSION=\"1.0\" DATE=\"$today\" OPERATEUR=\"$pi\" CODIFICATION=\"OOPC\">\n";
  print  XML_FILE "        <CODE>$CODE</CODE>\n";	    
  print  XML_FILE "        <COMMENTAIRE>$validation_comment</COMMENTAIRE>\n";
  print  XML_FILE "      </VALIDATION>\n";  
  print  XML_FILE "    </CYCLE_MESURE>\n";  
  print  XML_FILE "    <INSTRUMENT TYPE=\"$type\" NUMERO_SERIE=\"$sn\">\n"; 
  print  XML_FILE "    </INSTRUMENT>\n";  
  print  XML_FILE "  </ENTETE>\n";  
  print  XML_FILE "  <DATA>\n";  
}

#------------------------------------------------------------------------------
# entete ODV
#------------------------------------------------------------------------------
sub entete_odv { 
  my $today = &dateFormat(undef,"%d/%m/%Y");
  print  ODV_FILE "//ODV Spreadsheet file : $odv_file\n"; 
  print  ODV_FILE "//Data treated : $today\n"; 
  print  ODV_FILE "//\n"; 
  print  ODV_FILE "Cruise\tStation\tType\tmon/day/yr\thh:mm\tLon (E)\tLat (N)\tBot. Depth [m]\tDepth [m]\tTEMP [C]\tPSAL [Psu]\tDENS [kg/m3]\tSVEL [m/s]\n"; 
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
my ($d,$M,$Y,$h,$m,$s, $date2);

&dateInit( "EN","GMT" );
&usage if( $#ARGV == -1);
&get_options;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux. On peut se contenter de les mettre a jour uniquement dans
# ce fichier
open( HEADER, $ARGV[0] );
while( <HEADER> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/ or /Navio\s*:\s*(.*)/) {
    if ( not defined $plateforme ) {
      ($plateforme) = $1;
       chomp $plateforme;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ or /Comissao\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
}  
# nomme les fichiers
$hdr_file = lc $cycle_mesure.'.xbt';
$ascii_file = lc $cycle_mesure.'_xbt' if (defined $ascii);
$xml_file = $ascii_file.'.xml' if (defined $xml);
$odv_file = lc $cycle_mesure.'_xbt_odv.txt' if (defined $odv);

if( $DEBUG ) {
  printf "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
          $hdr_file,$ascii_file,$xml_file,$institute,$plateforme,$type,$sn,$pi;
  for( my $i = 0; $i <= $#ARGV; $i++ ){
    print $ARGV[$i] . "\n";
  }
  print "\nExit xbt-edf.pl ... debug mode\n";
  exit;
}
# ouverture des pointeurs (handles) de fichiers
if (defined $odv) {
  open( ODV_FILE,   "+> $odv_file" )   or die "Can't open file : $odv_file\n";
  &entete_odv;
} else {
  open( HDR_FILE,   "+> $hdr_file" )   or die "Can't open file : $hdr_file\n";
  open( ASCII_FILE, "+> $ascii_file" ) or die "Can't open file : $ascii_file\n";
  open( XML_FILE,   "+> $xml_file" )   or die "Can't open file : $xml_file\n";
}
# ecriture des entetes
print HDR_FILE "$cycle_mesure  $plateforme  $institute  $type  $sn  $pi\n";  
print HDR_FILE "St   Date  Heure    Latitude   Longitude  Profondeur\n";
if (defined $xml) {
  &entete_xml; 
  print XML_FILE   "PRFL   DEPH   TEMP   PSAL    DENS    SVEL \n";
}
if (defined $ascii) {
  print ASCII_FILE "$cycle_mesure  $plateforme  $institute  $type  $sn  $pi\n";  
  print ASCII_FILE "PRFL   DEPH   TEMP   PSAL    DENS    SVEL \n";
}  
close HEADER;

# parcourt les fichiers et cr une table de hashage numero_tir => nom_fichier
foreach (@ARGV) {
  ($key) = /_(\d+)/;
  $file{$key} = $_;
}

# tri le hash %file par numero_tir et parcourt le hash tri
foreach my $cle (sort { $a <=> $b } keys %file) { 
  open( DATA_FILE, $file{$cle} ) or warn("Erreur: " . $!);
  print STDERR  "Lit: $file{$cle}" if defined $ECHO;
  while( <DATA_FILE> ){           # header contient l'entete
    ($profil) = $1 if (/Sequence #\s+:\s+(\d+)/);
    if( /Probe Type\s+:\s+(.*)/)	{
      ($probe) = $1;
      chop $probe;
    }
    ($M,$d,$Y) = ($1,$2,$3) if (/Date of Launch:\s+(\d+)\/(\d+)\/(\d+)/);
    $year_ref = $Y if (!defined $year_ref);
    if (/Time of Launch:\s+(\d+):(\d+):(\d+)/)	{
      ($h,$m,$s) = ($1,$2,$3);	
      # transforme le day_of_year en julian day 0
      $julien = &date2julian($year_ref,$Y,$M,$d,$h,$m,$s);
      # a modifier absolument, verrue
      $date = sprintf("%02d/%02d/%04d %02d:%02d", $d,$M,$Y,$h,$m); # pour fichier entete	      
      $date2 = sprintf("%02d/%02d/%04d %02d:%02d", $M,$d,$Y,$h,$m); # pour fichier entete	      
    }
    if (/Latitude\s+:\s+(\d+)\s+(\d+\.\d+)(\w)/)	{
      ($lat_deg, $lat_min, $lat_hemi) = ($1,$2,$3);
      $lat_pos = &positionDeci($lat_deg, $lat_min, $lat_hemi);
    }
    if (/Longitude\s+:\s+(\d+)\s+(\d+\.\d+)(\w)/) {
      ($long_deg,$long_min,$long_hemi)= ($1,$2,$3); 
      $long_pos = &positionDeci($long_deg, $long_min, $long_hemi);
    }
    # detecte la fin de l'entete du fichier, a modifier eventuellement le test
    if (/This (XBT|XCTD) export/) {  
      printf XML_FILE "%3d  %4d %7.3f %7.4f %8.4f %s\n",$profil, $code,$julien,
          $lat_pos, $long_pos,&dateFormat($date2) if (defined $xml);
      printf ASCII_FILE "%3d  %4d %7.3f %7.4f %8.4f %s\n",$profil, $code,$julien,
          $lat_pos, $long_pos,&dateFormat($date2) if (defined $ascii);
    }
    # lit les donnees et eclate la ligne, utilise la ligne courante $_ par defaut
    if (/^\d/) {
      if ($probe =~ /XCTD/) {	    
        ($depth,$T,$C,$S,$sndvel,$sigmateta) = split;
      }	
      else {	      
        ($depth,$T,$sndvel) = split;
        $S = 35; 
	      # routine de calcul des parametres derives si necessaire, module seawater
	      #$T68 = $T0 * 1.00024; 
	      #$sndvel=&sw_svel($S,$T,$depth);
	      $sigmateta = &sw_sigmateta($S,$T,$depth);
        $S  = 1e36; 
      }	
      if ($T > 1.0) {
	      $dpth = $depth;  # memorise la profondeur max pour l'entete
        printf XML_FILE "%3d  %6.1f  %5.2f  %5.4g  %6.5g  %7.6g\n",
	      $profil, $depth, $T, $S, $sigmateta, $sndvel if (defined $xml);
        printf ASCII_FILE "%3d  %6.1f  %5.2f  %5.4g  %6.5g  %7.6g\n",
	      $profil, $depth, $T, $S, $sigmateta, $sndvel if (defined $ascii);
        if (defined $odv) {
					printf ODV_FILE "%s\t%3d\t%s\t%s\t%8.4f\t%7.4f\t%6.1f\t%6.1f",
	        $cycle_mesure, $profil, $type_odv, &dateFormat($date,"%m/%d/%Y\t%H:%M"),
					$long_pos, $lat_pos, $bottom_depth, $depth;
	        printf ODV_FILE ($T > 1e35) ? "\t" : "\t%5.2f", $T;
					printf ODV_FILE ($S > 1e35) ? "\t" : "\t%5.2f", $S;
					printf ODV_FILE "\t%7.2f",   $sndvel;
					printf ODV_FILE "\t%6.3f\n", $sigmateta;
	      }				
      }
    }
  }
  # ecriture de la ligne d'entete de chaque profil de le fichier .xbt 
  printf HDR_FILE  "%3d %s %02d%05.2f %s %03d%05.2f %s %4.0f %s\n",
          $profil, $date, $lat_deg, $lat_min, $lat_hemi, $long_deg, 
	  $long_min, $long_hemi, $dpth,$probe;
  printf STDERR "   %3d %s %02d%05.2f %s %03d%05.2f %s %4.0f %s\n",
          $profil, $date, $lat_deg, $lat_min, $lat_hemi, $long_deg, 
	  $long_min, $long_hemi, $dpth,$probe if defined $ECHO;
  $. = 0;
  close DATA_FILE;
}

# return output filename processed to matlab
(!defined $xml) ? print STDERR $ascii_file : print STDERR $xml_file;

# ferme les balises XML
if (defined $xml) {
  print  XML_FILE "  </DATA>\n";  
  print  XML_FILE "</OCEANO>\n";  
}  
# fermeture des pointeurs de fichiers
close HDR_FILE;
close XML_FILE;
close ASCII_FILE;
close ODV_FILE if defined $odv;

