#!c:\perl\bin\perl -w
#
# Traitement des fichiers SeaBird CTD acquis avec Seasave et traits avec Seasoft
# SBEProcessing. 
# Utilise les fichiers d'extensions .hdr et .asc 
# Les fichiers de sortie prennent le nom du cycle_mesure (entete) + extensions 
# et _ctd + extension eventuelle .xml
# Ce script lit et enregistre les capteurs primaires et secondaires
#
# J Grelet IRD july 2009
# 
# $Id$ 

use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
use IO::Handle;       # pour la creation des descripteur de fichier, perlfaq5.html
use Data::Dumper;
use Getopt::Long;
use Switch;
use seawater;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION = '1.2';
our $AUTHOR        = 'jgrelet IRD june 2009 PIRATA-FR19 cruise';
my  $DEBUG         = 0;
my  $ECHO;

# surcharge par la valeurs saisies sur la ligne de commande
my  $version;
my  $institute;
my  $type;
my  $sn;
my  $pi;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE      = "AMMA";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "20090614090000";  # 14/06/2009 a 09:00:00
my  $date_fin      = "20090723180000";  # 23/07/2009 a 18:00:00
my  $cruisePrefix  = "fr19";

my  $help;
my  $xml           = 1;
my  $ascii         = undef;
my  $dtd           = undef;

my  $code          = -1;  # code pour l'entete

my  $hdr_file;
my  $ctd_file;
my  $hdl_hdr = undef;     # handles de fichier

my($cycle_mesure, $ship) = undef;
my($seasave_version)     = undef;
my($station,$Pres,$T0,$T1,$FlC,$Bat,$Par,$Spar,$Cpar,$Ox0,$Ox1,$nbin,$S0,$S1);
my($sigmateta,$sndvel,$den1,$den2,$T68_1,$T68_2);
my($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_pos, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $time);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-fr19-all.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cycle_mesure=<name>  cycle_mesure name\n";
  print STDERR   "    --ship=<ship_name>     ship name\n";
  print STDERR   "    --institute=<name>     institute name\n";
  print STDERR   "    --code_oopc=<value>    processing code\n";
  print STDERR   "    --pi=<pi_name> \n";
  print STDERR   "    --ascii                ASCII output instead XML\n";
  print STDERR   "    --xml                  XML output (default)\n";
  print STDERR   "    --local                use local DTD\n";
  print STDERR   "    --sn=<serial_number>\n";
  print STDERR   "    --type=<instrument_type> \n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "example:\n\$ perl ctd-fr19-all.pl --cycle_mesure=PIRATA-FR19 --institute=IRD --ship='ANTEA' --sn=09P10828 --type=SBE911+ --pi=BOURLES data/fr19???.hdr --echo --local\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cycle_mesure=s"  => \$cycle_mesure,    
               "type=s"          => \$type,  
               "sn=s"            => \$sn,  
               "ship=s"          => \$ship,  
               "pi=s"            => \$pi,  
               "code_oopc=s"     => \$CODE,  	       
               "ascii"           => \$ascii,  
               "debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  	       
               "institute=s"     => \$institute, 
               "local"           => \$dtd,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
  $xml = undef if $ascii;    
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m) = @_;
  my $tmp = (($h * 60) + $m ) / 1440;
  return( $jj + $tmp ); 
}

#------------------------------------------------------------------------------
# voir documentation 
# http://www.seabird.com/pdf_documents/manuals/Seasave_5.37.pdf
#------------------------------------------------------------------------------
sub decode_con_file {
  my($fileName) = @_;
  my($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,$p_cond_cpcor);
  my($p_cond_sn,$p_cond_date,$p_cond_slope,$p_cond_offset);
  my($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j);
  my($p_temp_sn,$p_temp_date,$p_temp_slope,$p_temp_offset);

  # attention, verifier si les fichiers sont en majuscules ou minuscules !!!!
  $fileName =~ s/\.HDR/\.CON/i;
  open( DATA_CON, $fileName );
  while( <DATA_CON> ){           
    chomp;
    # chomp ne marche pas correctement car le fichier .con est au format DOS
    # avec un CRLF (^M^J ou OD OA) a la fin de chaque ligne
    # on vire le CR si present en fin de ligne
    $_ =~ s/^(.+)\r$/$1/;
    switch( $. ) {
       # conductivity	    
       case 1   { $p_cond_sn =  $_; }
       case 3   { (undef,undef,$p_cond_slope,$p_cond_offset) = split; }
       case 110 { ($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,$p_cond_cpcor) = split; }
       case 52  { $p_cond_date =  $_; }
       # temperature
       case 4   { $p_temp_sn =  $_; }
       case 5   { (undef,undef,undef,undef,undef,$p_temp_slope,$p_temp_offset) = split; }
       case 111 { ($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j) = split; }
       case 53  { $p_temp_date =  $_; }
    }  
  }
  close DATA_CON;

  #my $xml = qq|      
  #        <ETALONNAGE DATE=\"$p_cond_date\" CAPTEUR=\"COND\" NUMERO_SERIE=\"$p_cond_sn\" NUMERO=\"PRIMARY\">  
  #        <PARAMETRE VALEUR=\"G\">$p_cond_g</PARAMETRE>
  #|;
  #print CTD_FILE $xml;
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE04\" NUMERO_SERIE=\"$p_cond_sn\" CODE_ROSCOP=\"PSAL\" PERE=\"$sn\" NUMERO=\"PRIMARY\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_cond_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_cond_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_cond_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_cond_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CTCOR\" TYPE=\"N\">$p_cond_ctcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CPCOR\" TYPE=\"N\">$p_cond_cpcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_cond_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_cond_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE03\" NUMERO_SERIE=\"$p_temp_sn\" CODE_ROSCOP=\"TEMP\" PERE=\"$sn\" NUMERO=\"PRIMARY\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_temp_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"G\" TYPE=\"N\">$p_temp_g</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_temp_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_temp_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_temp_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"F0\" TYPE=\"N\">$p_temp_f0</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_temp_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_temp_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
}

#------------------------------------------------------------------------------
# entete XML
#------------------------------------------------------------------------------
sub entete_xml { 
  my($fileName) = @_;
  my $today = &UnixDate( &ParseDate("today"), "%d/%m/%Y");
  
  print  CTD_FILE '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  # les commentaires ne sont pas accepts par XML Toolbox Matlab de Geodise
  if ( defined $dtd ) {
		print  CTD_FILE '<!DOCTYPE OCEANO SYSTEM "/M/ACQUISIT/oceano.dtd">' . "\n";
		#print  CTD_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://antea/acquisit/oceano.dtd">' . "\n";
  } else {  
    print  CTD_FILE '<!DOCTYPE OCEANO PUBLIC "-//US025//DTD OCEANO//FR" "http://www.brest.ird.fr/us191/database/oceano.dtd">' . "\n";
  }
  print  CTD_FILE '<OCEANO TYPE="PROFIL">' . "\n";
  print  CTD_FILE "  <ENTETE>\n";
  print  CTD_FILE "    <PLATEFORME>\n";
  print  CTD_FILE "      <LIBELLE>$ship</LIBELLE>\n";  
  print  CTD_FILE "    </PLATEFORME>\n";
  print  CTD_FILE "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  CTD_FILE "      <LIBELLE>$cycle_mesure</LIBELLE>\n";  
  print  CTD_FILE "      <DATE_DEBUT>$date_debut</DATE_DEBUT>\n";  
  print  CTD_FILE "      <DATE_FIN>$date_fin</DATE_FIN>\n";  
  print  CTD_FILE "      <INSTITUT>$institute</INSTITUT>\n";  
  print  CTD_FILE "      <RESPONSABLE>$pi</RESPONSABLE>\n"; 
  print  CTD_FILE "      <ACQUISITION LOGICIEL=\"SEASAVE\" VERSION=\"$seasave_version\"></ACQUISITION>\n"; 
  print  CTD_FILE "      <TRAITEMENT LOGICIEL=\"SBEDATAPROCESSING\" VERSION=\"$seasave_version\"></TRAITEMENT>\n"; 
  print  CTD_FILE "      <VALIDATION LOGICIEL=\"$0\" VERSION=\"$VERSION\" DATE=\"$today\" OPERATEUR=\"$pi\" CODIFICATION=\"OOPC\">\n";
  print  CTD_FILE "        <CODE>$CODE</CODE>\n";	    
  print  CTD_FILE "        <COMMENTAIRE>Extraction ralise avant la post-calibration</COMMENTAIRE>\n";
  print  CTD_FILE "      </VALIDATION>\n";  
  print  CTD_FILE "    </CYCLE_MESURE>\n";  
  print  CTD_FILE "    <INSTRUMENT TYPE=\"$type\" NUMERO_SERIE=\"$sn\">\n"; 
  print  CTD_FILE "    </INSTRUMENT>\n";  
  
  &decode_con_file($fileName);
  
  print  CTD_FILE "  </ENTETE>\n";  
  print  CTD_FILE "  <DATA>\n";  
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

# variables locales
my $lines = 0;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $ship ) {
      ($ship) = $1;
       chop $ship;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($type) = $1  if ( not defined $type );
  }
  if( /Software Version Seasave Win32 V\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  

# ouverture des fichiers de sortie, on met en minuscule, lower case
my $fileName =  $ARGV[0];
my ($name,$dir) =  fileparse $fileName;
$ctd_file = lc $cycle_mesure.'-all_ctd';
$ctd_file .= '.xml' if defined $xml;
$hdr_file = lc  $cycle_mesure.'.ctd';

if( $DEBUG ) {
  printf "\nInput  file: %s\n", $hdr_file;
  printf "Output file: %s   mode: %s\n",$ctd_file, defined $xml ? 'XML' : 'ASCII'; 
  printf "%s\n%s\n%s\n%s\n%s\n",
          $institute,$ship,$type,$sn,$pi;
  for( my $i = 0; $i <= $#ARGV; $i++ ){
    print $ARGV[$i] . "\n";
  }
  print "\nExit ctd.pl ... debug mode\n";
  exit;
}
#$fh = IO::Handle->new();
open (CTD_FILE, "+> $ctd_file" ) or die "Can't open file : $ctd_file\n";
if (defined $hdl_hdr) {
  open (HDR_FILE , "+> $hdr_file" ) or die "Can't open file : $hdr_file\n";
  $hdl_hdr = *HDR_FILE;
}  

# ecriture des entetes
if ( defined $xml ) {
  &entete_xml( $fileName );
} else {	
  print CTD_FILE "$cycle_mesure  $ship  $institute  $type  $sn  $pi\n";
}

# Les paramtres supplmentaires
#DOX2;DISSOLVED OXYGEN;micromole/kg;0;450;%7.3lf;
#FLU2;FLUORESCENCE;milligram/m3;0;1;%8.4lf;
#TUR2;LIGHT ATTENUATION COEFFICIENT;m-1;0;10;%6.3lf;
#LGHT;LIGHT IRRADIANCE IMMERGED PAR;micromole photon/(m2.s);0;4000;%8.3lf;
#LGH4;LIGHT IRRADIANCE SURFACE PAR;micromole photon/(m2.s);0;3000;%8.3lf;
#LGH5;IMMERGED/SURF IRRADIANCE RATIO;%;0;100;%6.2lf;

print CTD_FILE "PRFL  PRES   TE01     TE02     PSA1     PSA2     DEN1    DEN2     DOX1     DOX2    NAVG\n";
print $hdl_hdr "$cycle_mesure  $ship  $institute  $type  $sn  $pi\n" if (defined $hdl_hdr);
print $hdl_hdr "St  Date   Heure    Latitude   Longitude  Profondeur\n\n" if (defined $hdl_hdr);
close DATA_FILE;

# parcourt des fichiers .HDR et .asc
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];
  open( DATA_FILE, $fileName );
  print STDERR  "Lit: $fileName" if defined $ECHO;
  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <DATA_FILE> ){           
    # les numeros de stations n'ont pas ete entres correctement dans les entetes
    # on utilise le numero du fichier correspondant
    if( $fileName =~ /.+$cruisePrefix(\d{3})/ ) {
      $station = $1;
    }  
    #if( /Station\s*:\s*(\d+)/ )	{
    #  ($station)  = $1; 
    #}
    # decodage de la position, date et heure entrees manuellement
    if( m[Date\s*:\s*(\d+)/(\d+)/(\d+)]) {
      ($jour,$mois,$annee) = ($1, $2, $3);	
    };	
    if( /Heure\s*:\s*(\d+)[:hH](\d+)/ ) {  # a modifier suivant le contexte
      ($heure,$min)  = ($1, $2);
      # conversion en format US (MM/DD/YY) pour ParseDate
      $date = $mois."/".$jour."/".$annee." ".$heure.":".$min.":00";
      #print STDERR  "Date: $date\n";
      $date = &ParseDate( $date );
      # transforme le day_of_year en julian day
      $julien = &UnixDate($time,"%j") -1;
      $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M") );
      $h_date=&UnixDate($date,"%H:%M %d/%m/%Y");
    }
    if( /Latitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /Longitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
    # decodage de la position, date et heure de l'entree NMEA (GPS)
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
    ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
    $date=&ParseDate($time);
    # transforme le day_of_year en julian day 0
    $julien = &UnixDate($time,"%j") -1;
    $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M") );
    $h_date = &UnixDate($date,"%H:%M %d/%m/%Y");
    }
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
  }
  
  printf CTD_FILE "%3d  %4d %7.3f %7.4f %8.4f %s 1e36 1e36 1e36 1e36 1e36\n",$station,
         $code,$julien,$lat_pos, $long_pos,&UnixDate($date,"%q");
  close DATA_FILE;
  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/i;
  open( DATA_FILE, $fileName );
  while( <DATA_FILE> ){           
    # on expurge toutes les lignes des entetes ainsi que les lignes vides
    # n'est pas necessaire avec asciiout
    if( not ( /^[*#]/ || /^\s*$/) ) {
      # lit les donnees, eclate la ligne,utilise la ligne courante $_ par defaut
      ($Pres,$T0,$T1,$Ox0,$Ox1,$nbin,$S0,$S1,$sigmateta,$sndvel) = split;
      # saute la premiere ligne
      next if( $Pres eq 'PrDM' );

      # recalcul le sigma-teta
      $T68_1 = $T0 * 1.00024; 
      $T68_2 = $T1 * 1.00024; 
      #$sndvel = &sw_svel($S0,$T68,$Pres);
      $den1 = &sw_sigmateta($S0,$T68_1,$Pres);
      $den2 = &sw_sigmateta($S1,$T68_2,$Pres);
      
      # mets toutes les valeurs inferieur a Pres, a mettre en longopt
      if ($lines == 0) {
	my $falseBin = 99999;      
	for( my $i = 0; $i <= $Pres; $i++ ) {
	  $falseBin = $nbin if ( $i == $Pres ); 
          printf CTD_FILE "%3d  %4d  %7.4f  %7.4f  %7.4f  %7.4f  %7.4f  %7.4f  %7.2f  %7.6g  %5d\n", $station,$i,$T0,$T1,$S0,$S1,$den1,$den2,$Ox0,$Ox1,$falseBin;
	}
      }
      else {
        printf CTD_FILE "%3d  %4d  %7.4f  %7.4f  %7.4f  %7.4f  %7.4f  %7.4f  %7.2f  %7.6g  %5d\n", $station,$Pres,$T0,$T1,$S0,$S1,$den1,$den2,$Ox0,$Ox1,$nbin;
      }
      $lines++;
    }
  }
  printf $hdl_hdr  "%03d %s %02d%05.2f %s %03d%05.2f %s %4.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $Pres if (defined $hdl_hdr);
  printf STDERR "   %03d %s %02d%05.2f %s %03d%05.2f %s %5.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $Pres if defined $ECHO;
  $Pres = 1e36;	  
  $. = 0; # remet le compteur de ligne a zero
  $lines = 0;
  close DATA_FILE;
}

# return result to matlab
print STDERR $ctd_file;

close $hdl_hdr if (defined $hdl_hdr) ;
if( defined $xml ) {
  print  CTD_FILE "  </DATA>\n";  
  print  CTD_FILE "</OCEANO>\n";  
}  
close CTD_FILE;

