#!/usr/bin/perl -w
#
# thermo-fr19.pl   J Grelet IRD june 2009
# $Id$
# analyse des fichiers thermo de l'Antea campagne PIRATA-FR19 14/06/2009 au 23/07/2009
# fichiers acquis avec seasave et traits avec sbe-dataprocessing 
# (.cnv incluant l'entete)
#
# usage ./thermo-fr19.pl   
#

use strict;
use Time::Local;
use Date::Manip;
use File::Basename;
use Getopt::Long;
use PDL;
use PDL::Math;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION = '1.0';
our $AUTHOR  = 'jgrelet IRD june 2009';
my  $DEBUG   = 0;
my  $ECHO;

# a mettre sous forme d'argument ligne de commande

# a changer !!!!!
my  $year = 2009;

my  $CYCLE_MESURE   = "PIRATA-FR19";
my  $SHIP           = "ANTEA";
my  $INSTITUTE      = "IRD";
my  $PI             = "BOURLES";
my  $INSTRUMENT_MTO = "BATOS";
my  $INSTRUMENT_TSG = "SBE21";
my  $INSTRUMENT_SN  = 3293;   # a lire dans le fichier
my  $CONTEXTE       = "AMMA";
my  $TIMEZONE       = "GMT";
my  $FORMAT_DATE    = "DMY";
my  $CODE           = "1A";
my  $date_debut     = "14/06/2009";
my  $date_fin       = "23/07/2009";

my  $softwareMeteo  = "BATOS";
my  $softwareMeteoVersion = "";
my  $softwareTsg    = undef; 
my  $softwareTsgVersion = "";

my  $version;
my  $help;
my  $xml           = 1;
my  $ascii         = undef;
my  $tsgqc         = undef;
my  $dtd           = undef;

# handles de fichiers
my  $tsg_dot_hdl   = undef;
my  $tsg_hdl       = undef;

my  $tsg_dot_file  = undef;
my  $tsg_file      = undef;

my  %file;
my  $key;

my ($j,$dif_met,$dif_sbe);
my ($julien,$time,$month,$day,$hour,$min,$sec);
my ($lat_s,$lat_deg,$lat_min,$lat_dec);
my ($long_s,$long_deg,$long_min,$long_dec);
my ($jmet,$jsbe);
# capteurs
# TSG
my ($scan,$T0,$T1,$C0,$S0,$P,$sigmateta,$sndvel);
# METEO
my ($date,$heure,$code,$lat,$long,$sonde,$tair,$tmer,$hum);
my ($patm,$rad,$rosee,$dir,$vit,$cond,$sbe38,$sss,$sst_cuve);
my $ecart;

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: perl thermo-fr19.pl [options]  <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --ascii                ASCII output instead XML\n";
  print STDERR   "    --tsgqc                ASCII output format used by validation software tsgqc\n";
  print STDERR   "    --xml                  XML output (default)\n";
  print STDERR   "    --local                use local DTD\n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "Thermo processing example:\n------------\n\$ perl thermo-fr19.pl --tsgqc --echo --local data/FR19_*.cnv\n\n"; 
  exit;
}


#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("ascii"           => \$ascii,  
               "xml"             => \$xml,  
               "tsgqc"           => \$tsgqc,  
               "debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  
               "local"           => \$dtd,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;	
  $xml = undef if $ascii or $tsgqc;   
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi)=@_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  return( ( $deg + ( $min / 60 ) ) * $sign ); 
}

sub julian {
  my($jj,$h,$m,$s)=@_;
  #print "$jj $h $m $s\n";
  my $tmp = ( ($h * 3600) + ( $m * 60 ) +$s ) / (1440 * 60);
  return( $jj + $tmp ); 
}

#------------------------------------------------------------------------------
# Conversion d'une heure ou position exprime en decimal en sexagesimal
#------------------------------------------------------------------------------
sub HtoHMS {
  my( $h1,$hemi ) = @_;

  my( $Sign,$h,$m1,$mi,$s);
  if ( $hemi eq 'lat') {
    if ( $h1 < 0 ) {
      $Sign = 'S';
    } else {
      $Sign = 'N';
    }
  } elsif( $hemi eq 'long' ) {
    if ( $h1 < 0 ) {
      $Sign = 'W';
    } else {
      $Sign = 'E';
    }   
  } 
  $h = $h1 < 0 ? ceil( $h1 ) : floor( $h1 );
  $m1 = ($h1 - $h) * 60;

  $mi = $m1 < 0 ? ceil( $m1 ) : floor( $m1 );
  $s = ($m1 - $mi) * 60;
  return( abs($h), abs($mi), abs($s), $Sign );
}   

#------------------------------------------------------------------------------
# entete XML
#------------------------------------------------------------------------------
sub entete_tsgqc { 
  my($handle) = @_;

  print $handle "%PLATFORM_NAME $SHIP\n";
  print $handle "%HEADER yyyy mm dd hh mm ss LATX LONX SSPS SSPS_QC SSPS_ADJUSTED SSPS_ADJUSTED_ERROR SSPS_ADJUSTED_QC SSJT SSJT_QC SSJT_ADJUSTED SSJT_ADJUSTED_ERROR SSJT_ADJUSTED_QC SSTP SSTP_QC SSTP_ADJUSTED SSTP_ADJUSTED_ERROR SSTP_ADJUSTED_QC\n";
  print $handle "%DATA\n";        
}

#------------------------------------------------------------------------------
# entete XML
#------------------------------------------------------------------------------
sub entete_xml { 
  my($handle,$software,$softVersion,$instrumentType,$INSTRUMENT_SN) = @_;
  my $today = &UnixDate( &ParseDate("today"), "%d/%m/%Y");

  print $handle '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  # les commentaires ne sont pas accepts par XML Toolbox Matlab de Geodise
  if ( defined $dtd ) {
    print  $handle '<!DOCTYPE OCEANO SYSTEM "M:\ACQUISIT\oceano.dtd">' . "\n";
    #print  $handle '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://antea/acquisit/oceano.dtd">' . "\n";
  } else {  
    print $handle '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://www.brest.ird.fr/us191/database/oceano.dtd">' . "\n";
  }
  print  $handle '<OCEANO TYPE="TRAJECTOIRE">' . "\n";
  print  $handle "  <ENTETE>\n";
  print  $handle "    <PLATEFORME>\n";
  print  $handle "      <LIBELLE>$SHIP</LIBELLE>\n";  
  print  $handle "    </PLATEFORME>\n";
  print  $handle "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  $handle "      <LIBELLE>$CYCLE_MESURE</LIBELLE>\n";  
  print  $handle "      <INSTITUT>$INSTITUTE</INSTITUT>\n";  
  print  $handle "      <RESPONSABLE>$PI</RESPONSABLE>\n"; 
  print  $handle "      <ACQUISITION LOGICIEL=\"$software\" VERSION=\"$softVersion\"></ACQUISITION>\n"; 
  print  $handle "      <TRAITEMENT LOGICIEL=\"$0\" VERSION=\"$VERSION\"></TRAITEMENT>\n"; 
  print  $handle "      <VALIDATION LOGICIEL=\"datagui\" VERSION=\"1.0\" DATE=\"$today\" OPERATEUR=\"$PI\" CODIFICATION=\"OOPC\">\n";
  print  $handle "        <CODE>$CODE</CODE>\n";	    
  print  $handle "        <COMMENTAIRE>Extraction ralise avant la post-calibration</COMMENTAIRE>\n";
  print  $handle "      </VALIDATION>\n";  
  print  $handle "    </CYCLE_MESURE>\n";  
  print  $handle "    <INSTRUMENT TYPE=\"$INSTRUMENT_TSG\" NUMERO_SERIE=\"$INSTRUMENT_SN\">\n"; 
  print  $handle "    </INSTRUMENT>\n";  
  print  $handle "  </ENTETE>\n";  
  print  $handle "  <DATA>\n";  
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $SHIP ) {
      ($SHIP) = $1;
       chop $SHIP;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($CYCLE_MESURE) = $1 if ( not defined $CYCLE_MESURE );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($INSTRUMENT_TSG) = $1  if ( not defined $INSTRUMENT_TSG );
  }
  if( /Software Version Seasave Win32 V\s+(\d+\.\d+)/ ) {
    ($softwareTsg) = $1  if ( not defined $softwareTsg );
  }
}  

# creation des noms de fichiers
$tsg_file = lc $CYCLE_MESURE.'_tsg';
$tsg_file .= '.xml' if defined $xml;
$tsg_dot_file = lc $CYCLE_MESURE.'.tsg' if (defined $ascii);
$tsg_dot_file = lc $CYCLE_MESURE.'.tsgqc' if (defined $tsgqc);

# ouverture des fichiers
if (defined $tsg_dot_file) {
  open( $tsg_dot_hdl,  "+> $tsg_dot_file" ) or die "Can't open file : $tsg_dot_file\n";
}
open( $tsg_hdl, "+> $tsg_file" )     or die "Can't open file : $tsg_file\n";

# ecriture des entetes
if (defined $xml) {
  &entete_xml($tsg_hdl,$softwareTsg, $softwareTsgVersion, $INSTRUMENT_TSG, "");
  print $tsg_hdl "YEAR     DAYD      LATX      LONX     SSJT    SSPS     CNDC     SSTP\n";
  #&entete_xml($mtod_hdl,$softwareMeteo, $softwareMeteoVersion, $INSTRUMENT_MTO, "");
  #print $mtod_hdl "YEAR     DAYD      LATX       LONX     SSTP  DRYT  WMSP  WDIR   ATMS  RELH    RDIN\n"; 
} 
else {
  print $tsg_hdl  "$CYCLE_MESURE  $SHIP  $INSTITUTE  $INSTRUMENT_TSG $INSTRUMENT_SN\n";
  print $tsg_hdl "YEAR     DAYD      LATX      LONX     SSJT    SSPS     CNDC     SSTP\n";
}

if (defined $tsgqc) {
  # dans tous les cas, on cree l'entete des fichiers au format ASCII tsgqc 
  &entete_tsgqc($tsg_dot_hdl);
}
if (defined $ascii) {
  print $tsg_dot_hdl  "$CYCLE_MESURE  $SHIP  $INSTITUTE  $INSTRUMENT_TSG $INSTRUMENT_SN\n";
  print $tsg_dot_hdl  " Julien     Date      Time     Latitude    Longitude    SSJT    SSPS     CNDC     SSTP\n";
}

$jmet = $jsbe = $dif_met = $dif_sbe = 0;
#$passage = 0;

# parcourt les fichiers et cr une table de hashage numero => nom_fichier
foreach (@ARGV) {
  ($key) = /_(\d+)/;
  $file{$key} = $_;
}

# parcourt des fichiers .cnv
# tri le hash %file par numero_tir et parcourt le hash tri
foreach my $cle (sort { $a <=> $b } keys %file) { 
  open( DATA_FILE, $file{$cle} ) or warn("Erreur: " . $!);
  print STDERR  "Lit: $file{$cle}\n" if defined $ECHO;
  while( <DATA_FILE> ){                  
    if( not ( /^[*#]/ || /^\s*$/) ) {
      ($scan,$T0,$C0,$T1,$lat_dec,$long_dec,$julien,$P,$S0,$sigmateta,$sndvel) = split;
      # Seabird utilise le julien 1, datemanip 1
      $julien = $julien -1;
      next if( $scan < 1 ); 					      
      #$T68 = $T0 * 1.00024; 
      #$sndvel=&sw_svel($S0,$T68,$P);
      #$sigmateta=&sw_sigmateta($S1,$T68,$P);
      # Returns the year, month, day, hour, minutes, and decimal seconds given
      # a floating point day of the year.
      ($year,$month,$day,$hour,$min,$sec) = &Date_NthDayOfYear($year,$julien+1);
      printf $tsg_hdl
          "%4d  %10.6f %+8.4f %+9.5f  %6.3f  %6.3f  %8.6f  %6.3f\n", 
           $year,$julien,$lat_dec,$long_dec,$T0,$S0,$C0,$T1;
      if (defined $tsgqc) {
        printf $tsg_dot_hdl  
          "%04d %02d %02d %02d %02d %02d   %+10.7f   %+11.7f %6.3f 0    NaN    NaN 0 %6.3f 0    NaN    NaN 0  %6.3f 0   NaN   NaN 0\n", 
          $year,$month,$day,$hour,$min,$sec,$lat_dec,$long_dec,
          $S0,$T0,$T1;
      }
      if (defined $ascii) {
        printf $tsg_dot_hdl  
          "%7.4f %02d/%02d/%04d %02d:%02d:%02d  %2d%02d.%02d %s  %3d%02d.%02d %s  %6.3f  %6.3f  %8.6f  %6.3f\n", 
           $julien,$day,$month,$year,$hour,$min,$sec,
 	   &HtoHMS($lat_dec,'lat'),&HtoHMS($long_dec,'long'),
	   $T0,$S0,$C0,$T1;
      }
    }
  }

  $. = 0; # remet le compteur de ligne a zero
  close DATA_FILE;
}

if( defined $xml ) {
  print  $tsg_hdl "  </DATA>\n</OCEANO>\n";  
} 

close $tsg_hdl;
if (defined $tsgqc or defined $ascii) {
  close $tsg_dot_hdl;
}
